@@ -1,70 +1,78 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.transaction import Transaction
+
+__all__ = ['Country', 'Subdivision']
 
 
 class Country(ModelSQL, ModelView):
     'Country'
-    _name = 'country.country'
-    _description = __doc__
+    __name__ = 'country.country'
     name = fields.Char('Name', required=True, translate=True,
-           help='The full name of the country.', select=1)
-    code = fields.Char('Code', size=2, select=1,
+           help='The full name of the country.', select=True)
+    code = fields.Char('Code', size=2, select=True,
            help='The ISO country code in two chars.\n'
            'You can use this field for quick search.', required=True)
     subdivisions = fields.One2Many('country.subdivision',
             'country', 'Subdivisions')
 
-    def __init__(self):
-        super(Country, self).__init__()
-        self._sql_constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(Country, cls).__setup__()
+        cls._sql_constraints += [
             ('name_uniq', 'UNIQUE(name)',
-             'The name of the country must be unique!'),
+                'The name of the country must be unique.'),
             ('code_uniq', 'UNIQUE(code)',
-             'The code of the country must be unique!'),
-        ]
-        self._order.insert(0, ('name', 'ASC'))
+                'The code of the country must be unique.'),
+            ]
+        cls._order.insert(0, ('name', 'ASC'))
 
-    def search_rec_name(self, name, clause):
-        ids = self.search([('code',) + clause[1:]], limit=1)
-        if ids:
-            return [('code',) + clause[1:]]
-        return [(self._rec_name,) + clause[1:]]
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        if cls.search([('code',) + tuple(clause[1:])], limit=1):
+            return [('code',) + tuple(clause[1:])]
+        return [(cls._rec_name,) + tuple(clause[1:])]
 
-    def create(self, vals):
-        if 'code' in vals and vals['code']:
-            vals = vals.copy()
-            vals['code'] = vals['code'].upper()
-        return super(Country, self).create(vals)
+    @classmethod
+    def create(cls, vlist):
+        vlist = [x.copy() for x in vlist]
+        for vals in vlist:
+            if 'code' in vals and vals['code']:
+                vals['code'] = vals['code'].upper()
+        return super(Country, cls).create(vlist)
 
-    def write(self, ids, vals):
-        if 'code' in vals and vals['code']:
-            vals = vals.copy()
-            vals['code'] = vals['code'].upper()
-        return super(Country, self).write(ids, vals)
-
-Country()
+    @classmethod
+    def write(cls, *args):
+        actions = iter(args)
+        args = []
+        for countries, values in zip(actions, actions):
+            if values.get('code'):
+                values = values.copy()
+                values['code'] = values['code'].upper()
+            args.extend((countries, values))
+        super(Country, cls).write(*args)
 
 
 class Subdivision(ModelSQL, ModelView):
     "Subdivision"
-    _name = 'country.subdivision'
-    _description = __doc__
+    __name__ = 'country.subdivision'
     country = fields.Many2One('country.country', 'Country',
-            required=True, select=1)
-    name = fields.Char('Name', required=True, select=1)
-    code = fields.Char('Code', required=True, select=1)
+            required=True, select=True)
+    name = fields.Char('Name', required=True, select=True, translate=True)
+    code = fields.Char('Code', required=True, select=True)
     type = fields.Selection([
         ('administration', 'Administration'),
         ('administrative area', 'Administrative area'),
+        ('administrative atoll', 'Administrative atoll'),
         ('administrative region', 'Administrative Region'),
         ('administrative territory', 'Administrative Territory'),
         ('area', 'Area'),
         ('atoll', 'Atoll'),
+        ('arctic region', 'Arctic Region'),
         ('autonomous city', 'Autonomous City'),
         ('autonomous commune', 'Autonomous Commune'),
         ('autonomous communities', 'Autonomous communities'),
+        ('autonomous community', 'Autonomous community'),
         ('autonomous district', 'Autonomous District'),
         ('autonomous island', 'Autonomous island'),
         ('autonomous monastic state', 'Autonomous monastic state'),
@@ -74,21 +82,27 @@ class Subdivision(ModelSQL, ModelView):
         ('autonomous republic', 'Autonomous republic'),
         ('autonomous sector', 'Autonomous sector'),
         ('autonomous territory', 'Autonomous territory'),
+        ('autonomous territorial unit', 'Autonomous territorial unit'),
         ('borough', 'Borough'),
         ('canton', 'Canton'),
         ('capital city', 'Capital city'),
         ('capital district', 'Capital District'),
         ('capital metropolitan city', 'Capital Metropolitan City'),
         ('capital territory', 'Capital Territory'),
+        ('chains (of islands)', 'Chains (of islands)'),
         ('city', 'City'),
         ('city corporation', 'City corporation'),
         ('city with county rights', 'City with county rights'),
         ('commune', 'Commune'),
+        ('constitutional province', 'Constitutional province'),
+        ('council area', 'Council area'),
         ('country', 'Country'),
         ('county', 'County'),
         ('department', 'Department'),
         ('dependency', 'Dependency'),
+        ('development region', 'Development region'),
         ('district', 'District'),
+        ('district council area', 'District council area'),
         ('division', 'Division'),
         ('economic prefecture', 'Economic Prefecture'),
         ('economic region', 'Economic region'),
@@ -97,10 +111,13 @@ class Subdivision(ModelSQL, ModelView):
         ('federal dependency', 'Federal Dependency'),
         ('federal district', 'Federal District'),
         ('federal territories', 'Federal Territories'),
+        ('geographical entity', 'Geographical entity'),
         ('geographical region', 'Geographical region'),
         ('geographical unit', 'Geographical unit'),
         ('governorate', 'Governorate'),
         ('included for completeness', 'Included for completeness'),
+        ('indigenous region', 'Indigenous region'),
+        ('island', 'Island'),
         ('island council', 'Island council'),
         ('island group', 'Island group'),
         ('local council', 'Local council'),
@@ -114,18 +131,24 @@ class Subdivision(ModelSQL, ModelView):
         ('oblast', 'Oblast'),
         ('outlying area', 'Outlying area'),
         ('overseas region/department', 'Overseas region/department'),
-        ('overseas territorial collectivity', 'Overseas territorial collectivity'),
+        ('overseas territorial collectivity',
+            'Overseas territorial collectivity'),
         ('parish', 'Parish'),
+        ('popularates', 'Popularates'),
         ('prefecture', 'Prefecture'),
         ('principality', 'Principality'),
         ('province', 'Province'),
+        ('quarter', 'Quarter'),
         ('rayon', 'Rayon'),
         ('region', 'Region'),
         ('regional council', 'Regional council'),
         ('republic', 'Republic'),
+        ('republican city', 'Republican City'),
+        ('self-governed part', 'Self-governed part'),
         ('special administrative region', 'Special administrative region'),
         ('special city', 'Special city'),
         ('special district', 'Special District'),
+        ('special island authority', 'Special island authority'),
         ('special municipality', 'Special Municipality'),
         ('special region', 'Special Region'),
         ('special zone', 'Special zone'),
@@ -136,27 +159,38 @@ class Subdivision(ModelSQL, ModelView):
         ('two-tier county', 'Two-tier county'),
         ('union territory', 'Union territory'),
         ('unitary authority', 'Unitary authority'),
+        ('unitary authority (england)', 'Unitary authority (england)'),
+        ('unitary authority (wales)', 'Unitary authority (wales)'),
+        ('zone', 'zone'),
         ], 'Type', required=True)
     parent = fields.Many2One('country.subdivision', 'Parent')
 
-    def __init__(self):
-        super(Subdivision, self).__init__()
-        self._order.insert(0, ('code', 'ASC'))
+    @classmethod
+    def __setup__(cls):
+        super(Subdivision, cls).__setup__()
+        cls._order.insert(0, ('code', 'ASC'))
 
-    def search_rec_name(self, name, clause):
-        ids = self.search([('code',) + clause[1:]], limit=1)
-        if ids:
-            return [('code',) + clause[1:]]
-        return [(self._rec_name,) + clause[1:]]
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        if cls.search([('code',) + tuple(clause[1:])], limit=1):
+            return [('code',) + tuple(clause[1:])]
+        return [(cls._rec_name,) + tuple(clause[1:])]
 
-    def create(self, vals):
-        if 'code' in vals and vals['code']:
-            vals['code'] = vals['code'].upper()
-        return super(Subdivision, self).create(vals)
+    @classmethod
+    def create(cls, vlist):
+        vlist = [x.copy() for x in vlist]
+        for vals in vlist:
+            if 'code' in vals and vals['code']:
+                vals['code'] = vals['code'].upper()
+        return super(Subdivision, cls).create(vlist)
 
-    def write(self, ids, vals):
-        if 'code' in vals and vals['code']:
-            vals['code'] = vals['code'].upper()
-        return super(Subdivision, self).write(ids, vals)
-
-Subdivision()
+    @classmethod
+    def write(cls, *args):
+        actions = iter(args)
+        args = []
+        for subdivisions, values in zip(actions, actions):
+            if values.get('code'):
+                values = values.copy()
+                values['code'] = values['code'].upper()
+            args.extend((subdivisions, values))
+        super(Subdivision, cls).write(*args)
